@@ -1,11 +1,12 @@
 #![doc = include_str!("../README.md")]
 #![deny(rust_2018_idioms)]
 #![deny(missing_docs)]
-#![deny(rustdoc::missing_doc_code_examples)]
+#![deny(rustdoc::all)]
 
 // this is vendored code from the `futures-rs` crate, to avoid
 // having a huge dependency when we only need a little bit
 mod futures;
+use crate::futures::{enter::enter, waker_ref, ArcWake, FuturesUnordered};
 
 use std::{
     collections::VecDeque,
@@ -23,10 +24,6 @@ use std::{
     thread::{self, Thread},
 };
 
-use futures::arc_wake::ArcWake;
-
-use crate::futures::{enter::enter, waker_ref, FuturesUnordered};
-
 /// A single-threaded, sequential, parameterized async task queue.
 ///
 /// This executor allows you to queue multiple tasks in sequence, and to
@@ -37,138 +34,14 @@ use crate::futures::{enter::enter, waker_ref, FuturesUnordered};
 /// and calling [queue](CosyncQueueHandle::queue), or, within a task, calling
 /// [queue_task](CosyncInput::queue) on [CosyncInput].
 #[derive(Debug)]
-pub struct Cosync<T> {
+pub struct Cosync<T: ?Sized> {
     pool: FuturesUnordered<FutureObject>,
     incoming: Arc<Mutex<VecDeque<FutureObject>>>,
     data: Box<Option<NonNull<T>>>,
     kill_box: Arc<()>,
 }
 
-/// A handle to spawn tasks.
-///
-/// # Examples
-/// ```
-/// # use cosync::Cosync;
-/// let mut cosync = Cosync::new();
-/// let handler = cosync.create_queue_handle();
-///
-/// // make a thread and join it...
-/// std::thread::spawn(move || {
-///     handler.queue(|mut input| async move {
-///         *input.get() = 20;
-///     });
-/// })
-/// .join()
-/// .unwrap();
-///
-/// let mut value = 1;
-/// cosync.run_blocking(&mut value);
-/// assert_eq!(value, 20);
-/// ```
-#[derive(Debug)]
-pub struct CosyncQueueHandle<T> {
-    heap_ptr: *const Option<NonNull<T>>,
-    incoming: Arc<Mutex<VecDeque<FutureObject>>>,
-    kill_box: Weak<()>,
-}
-
-impl<T: 'static> CosyncQueueHandle<T> {
-    /// Adds a new Task to the TaskQueue.
-    pub fn queue<Task, Out>(&self, task: Task)
-    where
-        Task: FnOnce(CosyncInput<T>) -> Out + Send + 'static,
-        Out: Future<Output = ()> + Send,
-    {
-        queue_task(task, self.kill_box.clone(), self.heap_ptr, &self.incoming);
-    }
-}
-
-// safety:
-// we guarantee with a kill counter that the main `.get` of CosyncInput
-// never dereferences invalid data, and it's only made in the same thread
-// as Cosync, so we should never have a problem with multithreaded access
-// at the same time.
-#[allow(clippy::non_send_fields_in_send_ty)]
-unsafe impl<T> Send for CosyncQueueHandle<T> {}
-unsafe impl<T> Sync for CosyncQueueHandle<T> {}
-
-impl<T> Clone for CosyncQueueHandle<T> {
-    fn clone(&self) -> Self {
-        Self {
-            heap_ptr: self.heap_ptr,
-            incoming: self.incoming.clone(),
-            kill_box: self.kill_box.clone(),
-        }
-    }
-}
-
-/// A guarded pointer to create a [CosyncInputGuard] by [get] and to queue more tasks by [queue]
-///
-/// [queue]: Self::queue
-/// [get]: Self::get
-#[derive(Debug)]
-pub struct CosyncInput<T>(CosyncQueueHandle<T>);
-
-impl<T: 'static> CosyncInput<T> {
-    /// Gets the underlying [CosyncInputGuard].
-    pub fn get(&mut self) -> CosyncInputGuard<'_, T> {
-        // if you find this guard, it means that you somehow moved the `CosyncInput` out of
-        // the closure, and then dropped the `Cosync`. Why would you do that? Don't do that.
-        assert!(
-            Weak::strong_count(&self.0.kill_box) == 1,
-            "cosync was dropped improperly"
-        );
-
-        // we can always dereference this data, as we maintain
-        // that it's always present.
-        let o = unsafe {
-            (&*self.0.heap_ptr)
-                .expect("cosync was not initialized this run correctly")
-                .as_mut()
-        };
-
-        CosyncInputGuard(o, PhantomData)
-    }
-
-    /// Queues a new task. This goes to the back of queue.
-    pub fn queue<Task, Out>(&self, task: Task)
-    where
-        Task: Fn(CosyncInput<T>) -> Out + Send + 'static,
-        Out: Future<Output = ()> + Send,
-    {
-        self.0.queue(task)
-    }
-}
-
-// safety:
-// we create `CosyncInput` per task, and it doesn't escape our closure.
-// therefore, it's `*const` field should only be accessible when we know
-// it's valid.
-#[allow(clippy::non_send_fields_in_send_ty)]
-unsafe impl<T> Send for CosyncInput<T> {}
-unsafe impl<T> Sync for CosyncInput<T> {}
-
-/// A guarded pointer.
-///
-/// This exists to prevent holding onto the `CosyncInputGuard` over `.await` calls. It will need to
-/// be fetched again from [CosyncInput] after awaits.
-pub struct CosyncInputGuard<'a, T>(&'a mut T, PhantomData<*const u8>);
-
-impl<'a, T> ops::Deref for CosyncInputGuard<'a, T> {
-    type Target = T;
-
-    fn deref(&self) -> &Self::Target {
-        self.0
-    }
-}
-
-impl<'a, T> ops::DerefMut for CosyncInputGuard<'a, T> {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        self.0
-    }
-}
-
-impl<T: 'static> Cosync<T> {
+impl<T: 'static + ?Sized> Cosync<T> {
     /// Create a new, empty queue of tasks.
     pub fn new() -> Self {
         Self {
@@ -179,6 +52,28 @@ impl<T: 'static> Cosync<T> {
         }
     }
 
+    /// Returns the number of tasks queued. This *includes* the task currently being executed. Use
+    /// [is_executing] to see if there is a task currently being executed (ie, it returned `Pending`
+    /// at some point in its execution).
+    ///
+    /// [is_executing]: Self::is_executing
+    pub fn len(&self) -> usize {
+        let one = if self.is_executing() { 1 } else { 0 };
+
+        one + self.incoming.lock().unwrap().len()
+    }
+
+    /// Returns true if no futures are being executed *and* there are no futures in the queue.
+    pub fn is_empty(&self) -> bool {
+        !self.is_executing() && self.incoming.lock().unwrap().is_empty()
+    }
+
+    /// Returns true if `cosync` has a `Pending` future. It is possible for
+    /// the `cosync` to have no `Pending` future, but to have tasks queued still.
+    pub fn is_executing(&self) -> bool {
+        !self.pool.is_empty()
+    }
+
     /// Creates a queue handle which can be used to spawn tasks.
     pub fn create_queue_handle(&self) -> CosyncQueueHandle<T> {
         let heap_ptr = &*self.data as *const Option<_>;
@@ -304,6 +199,130 @@ impl<T: 'static> Cosync<T> {
     }
 }
 
+/// A handle to spawn tasks.
+///
+/// # Examples
+/// ```
+/// # use cosync::Cosync;
+/// let mut cosync = Cosync::new();
+/// let handler = cosync.create_queue_handle();
+///
+/// // make a thread and join it...
+/// std::thread::spawn(move || {
+///     handler.queue(|mut input| async move {
+///         *input.get() = 20;
+///     });
+/// })
+/// .join()
+/// .unwrap();
+///
+/// let mut value = 1;
+/// cosync.run_blocking(&mut value);
+/// assert_eq!(value, 20);
+/// ```
+#[derive(Debug)]
+pub struct CosyncQueueHandle<T: ?Sized> {
+    heap_ptr: *const Option<NonNull<T>>,
+    incoming: Arc<Mutex<VecDeque<FutureObject>>>,
+    kill_box: Weak<()>,
+}
+
+impl<T: 'static + ?Sized> CosyncQueueHandle<T> {
+    /// Adds a new Task to the TaskQueue.
+    pub fn queue<Task, Out>(&self, task: Task)
+    where
+        Task: FnOnce(CosyncInput<T>) -> Out + Send + 'static,
+        Out: Future<Output = ()> + Send,
+    {
+        queue_task(task, self.kill_box.clone(), self.heap_ptr, &self.incoming);
+    }
+}
+
+// safety:
+// we guarantee with a kill counter that the main `.get` of CosyncInput
+// never dereferences invalid data, and it's only made in the same thread
+// as Cosync, so we should never have a problem with multithreaded access
+// at the same time.
+#[allow(clippy::non_send_fields_in_send_ty)]
+unsafe impl<T> Send for CosyncQueueHandle<T> {}
+unsafe impl<T> Sync for CosyncQueueHandle<T> {}
+
+impl<T> Clone for CosyncQueueHandle<T> {
+    fn clone(&self) -> Self {
+        Self {
+            heap_ptr: self.heap_ptr,
+            incoming: self.incoming.clone(),
+            kill_box: self.kill_box.clone(),
+        }
+    }
+}
+
+/// A guarded pointer to create a [CosyncInputGuard] by [get] and to queue more tasks by [queue]
+///
+/// [queue]: Self::queue
+/// [get]: Self::get
+#[derive(Debug)]
+pub struct CosyncInput<T: ?Sized>(CosyncQueueHandle<T>);
+
+impl<T: 'static + ?Sized> CosyncInput<T> {
+    /// Gets the underlying [CosyncInputGuard].
+    pub fn get(&mut self) -> CosyncInputGuard<'_, T> {
+        // if you find this guard, it means that you somehow moved the `CosyncInput` out of
+        // the closure, and then dropped the `Cosync`. Why would you do that? Don't do that.
+        assert!(
+            Weak::strong_count(&self.0.kill_box) == 1,
+            "cosync was dropped improperly"
+        );
+
+        // we can always dereference this data, as we maintain
+        // that it's always present.
+        let o = unsafe {
+            (&*self.0.heap_ptr)
+                .expect("cosync was not initialized this run correctly")
+                .as_mut()
+        };
+
+        CosyncInputGuard(o, PhantomData)
+    }
+
+    /// Queues a new task. This goes to the back of queue.
+    pub fn queue<Task, Out>(&self, task: Task)
+    where
+        Task: Fn(CosyncInput<T>) -> Out + Send + 'static,
+        Out: Future<Output = ()> + Send,
+    {
+        self.0.queue(task)
+    }
+}
+
+// safety:
+// we create `CosyncInput` per task, and it doesn't escape our closure.
+// therefore, it's `*const` field should only be accessible when we know
+// it's valid.
+#[allow(clippy::non_send_fields_in_send_ty)]
+unsafe impl<T: ?Sized> Send for CosyncInput<T> {}
+unsafe impl<T: ?Sized> Sync for CosyncInput<T> {}
+
+/// A guarded pointer.
+///
+/// This exists to prevent holding onto the `CosyncInputGuard` over `.await` calls. It will need to
+/// be fetched again from [CosyncInput] after awaits.
+pub struct CosyncInputGuard<'a, T: ?Sized>(&'a mut T, PhantomData<*const u8>);
+
+impl<'a, T: ?Sized> ops::Deref for CosyncInputGuard<'a, T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        self.0
+    }
+}
+
+impl<'a, T: ?Sized> ops::DerefMut for CosyncInputGuard<'a, T> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        self.0
+    }
+}
+
 impl<T: 'static> Default for Cosync<T> {
     fn default() -> Self {
         Self::new()
@@ -406,7 +425,7 @@ fn poll_executor<T, F: FnMut(&mut Context<'_>) -> T>(mut f: F) -> T {
 }
 
 /// Adds a new Task to the TaskQueue.
-fn queue_task<T: 'static, Task, Out>(
+fn queue_task<T: 'static + ?Sized, Task, Out>(
     task: Task,
     kill_box: Weak<()>,
     heap_ptr: *const Option<NonNull<T>>,
@@ -487,6 +506,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::needless_late_init)]
     fn pool_is_sequential() {
         // notice that value is declared here
         let mut value;
@@ -545,6 +565,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::needless_late_init)]
     fn pool_remains_sequential() {
         // notice that value is declared here
         let mut value;
@@ -568,6 +589,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::needless_late_init)]
     fn pool_is_still_sequential() {
         // notice that value is declared here
         let mut value;
@@ -598,6 +620,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::needless_late_init)]
     fn cosync_can_be_moved() {
         // notice that value is declared here
         let mut value;
@@ -669,4 +692,52 @@ mod tests {
         let t = trybuild::TestCases::new();
         t.compile_fail("tests/try_build/*.rs");
     }
+
+    #[test]
+    fn dynamic_dispatch() {
+        trait DynDispatch {
+            fn test(&self) -> i32;
+        }
+
+        impl DynDispatch for i32 {
+            fn test(&self) -> i32 {
+                *self
+            }
+        }
+
+        impl DynDispatch for &'static str {
+            fn test(&self) -> i32 {
+                self.parse().unwrap()
+            }
+        }
+
+        let mut cosync: Cosync<dyn DynDispatch> = Cosync::new();
+        cosync.queue(|mut input: CosyncInput<dyn DynDispatch>| async move {
+            {
+                let inner: &mut dyn DynDispatch = &mut *input.get();
+                assert_eq!(inner.test(), 3);
+            }
+
+            sleep_ticks(1).await;
+
+            {
+                let inner: &mut dyn DynDispatch = &mut *input.get();
+                assert_eq!(inner.test(), 3);
+            }
+        });
+
+        cosync.run_until_stall(&mut 3);
+        cosync.run_until_stall(&mut "3");
+    }
+
+    #[test]
+    fn unsized_type() {
+        let mut cosync: Cosync<str> = Cosync::new();
+
+        cosync.queue(|mut input| async move {
+            let input_guard = input.get();
+            let inner_str: &str = &input_guard;
+            println!("inner str = {}", inner_str);
+        });
+    }
 }
